<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: Common.php 3454 2005-07-15 13:56:17Z andrew $
*/

require_once MAX_PATH . '/lib/Max.php';
require_once MAX_PATH . '/lib/core/ServiceLocator.php';
require_once MAX_PATH . '/lib/max/Dal/Statistics.php';
require_once MAX_PATH . '/lib/max/Dal/Statistics/Common.php';
require_once 'Date.php';

/**
 * An abstract class that defines the interface and some common methods for
 * summarising raw data into statistics, for all Max modules.
 *
 * @package    MaxMaintenance
 * @subpackage Statistics
 * @author     Andrew Hill <andrew@m3.net>
 * @abstract
 */
class Maintenance_Statistics_Common
{
    var $module;
    var $now;
    var $dal;
    var $report;
    var $updateUsingOI;
    var $lastDateOI;
    var $updateOI;
    var $upToDateOI;
    var $lastDateHourly;
    var $updateHourly;    
    var $upToDateHourly;
    
    /**
     * A common constructor method that can be used for all children of the class.
     * Must have the class $module variable set before being called in the child
     * constructor method.
     */
    function Maintenance_Statistics_Common()
    {
        // Create the DAL class needed, and register it in the service locator
        $oMDS = &new MAX_Dal_Statistics();
        $oMaxDalStatistics = $oMDS->instance($this->module);
        $oServiceLocator = &ServiceLocator::instance();
        $oServiceLocator->register('MAX_Dal_Statistics_' . $this->module, $oMaxDalStatistics);
    }
    
    /**
     * A common method that can be used in all children of the class for determining
     * if the statistics for the module need to be updated, or not.
     *
     * @param Date $now Optional Date representing the current date/time. Normally
     *                  just used for testing the class constructor, or for 
     *                  faking the current time in order to re-generate past statistics,
     *                  in the case of faulty raw data tables.
     * @param Date $lastUpdated Optional Date representing the date/time the
     *                          statistics were last updated. Only used when
     *                          regenerating statistics.
     */
    function setUpdateRequirements($now = null, $lastUpdated = null)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $this->now = &new Date();
        if (!is_null($now)) {
            MAX::debug('Using supplied current date, instead of current time', PEAR_LOG_DEBUG);
            $this->now->copy($now);
        }
        $oServiceLocator = &ServiceLocator::instance();
        $this->dal = &$oServiceLocator->get('MAX_Dal_Statistics_' . $this->module);
        $this->report  = "Maintenance Statistics Report: $this->module Module\n";
        MAX::debug("Running Maintenance Statistics for the $this->module module", PEAR_LOG_INFO);
        $this->report .= "=====================================\n\n";
        $this->report .= 'Current time is ' . $this->now->format('%Y-%m-%d %H:%M:%S') . ".\n";
        MAX::debug('Current time is ' . $this->now->format('%Y-%m-%d %H:%M:%S'), PEAR_LOG_DEBUG);
        // Which of the operation interval and an hour is smaller?
        if ($conf['maintenance']['operationInterval'] <= 60) {
            $this->updateUsingOI = true;
        } else {
            $this->updateUsingOI = false;
        }
        // Don't update unless the time is right!
        $this->updateOI     = false;
        $this->updateHourly = false;
        // Determine when the last operation interval update happened
        if (is_null($lastUpdated)) {
            $this->lastDateOI =
                $this->dal->getMaintenanceStatisticsLastRunInfo(DAL_STATISTICS_COMMON_UPDATE_OI, $now);
        } else {
            $this->lastDateOI = $lastUpdated;
        }
        if (is_null($this->lastDateOI)) {
            $this->report .= "Maintenance statistics has never been run before, and there is no raw data\n" .
                             "in the database, so maintenance statistics will not be run for the operation interval.\n\n";
            $message = "Maintenance statistics has never been run before, and there is no raw data " .
                       "in the database, so maintenance statistics will not be run for the operation interval";
            MAX::debug($message, PEAR_LOG_DEBUG);
        } else {
            $this->report .= "Maintenance statistics last updated OI statistics to " . 
                             $this->lastDateOI->format('%Y-%m-%d %H:%M:%S') . ".\n";
            $message = "Maintenance statistics last updated OI statistics to " . 
                       $this->lastDateOI->format('%Y-%m-%d %H:%M:%S') . ".";
            MAX::debug($message, PEAR_LOG_DEBUG);
            $requiredDate = &new Date();
            $requiredDate->copy($this->lastDateOI);
            $requiredDate->addSeconds($conf['maintenance']['operationInterval'] * 60);
            $this->report .= 'Current time must be after ' . $requiredDate->format('%Y-%m-%d %H:%M:%S') . 
                             " for the next operation interval update to happen.\n";
            $message = 'Current time must be after ' . $requiredDate->format('%Y-%m-%d %H:%M:%S') . 
                       ' for the next operation interval update to happen';
            MAX::debug($message, PEAR_LOG_DEBUG);
            if (Date::compare($this->now, $requiredDate) > 0) {
                $this->updateOI = true;
                // Update OI stats to the end of the previous (not current) operation interval
                list($this->upToDateOI, $_) = Maintenance::convertDateToOperationIntervalStartAndEndDates($this->now);
                $this->upToDateOI->subtractSeconds(1);
            } else {
                // An operation interval hasn't passed, so don't update
                $this->report .= "At least {$conf['maintenance']['operationInterval']} minutes have " .
                                 "not passed since the last operation interval update.\n";
                $message = "At least {$conf['maintenance']['operationInterval']} minutes have " .
                           'not passed since the last operation interval update';
                MAX::debug($message, PEAR_LOG_DEBUG);
            }
        }
        // Determine when the last hourly update happened
        if (is_null($lastUpdated)) {
            $this->lastDateHourly =
                $this->dal->getMaintenanceStatisticsLastRunInfo(DAL_STATISTICS_COMMON_UPDATE_HOUR, $now);
        } else {
            $this->lastDateHourly = $lastUpdated;
        }
        if (is_null($this->lastDateHourly)) {
            // There are no statistics, cannot run
            $this->report .= "Maintenance statistics has never been run before, and there is no raw data\n" .
                             "in the database, so maintenance statistics will not be run for the hour.\n\n";
            $message = "Maintenance statistics has never been run before, and there is no raw data " .
                       "in the database, so maintenance statistics will not be run for the hour";
            MAX::debug($message, PEAR_LOG_DEBUG);
        } else {
            $this->report .= 'Maintenance statistics last updated hourly statistics to ' .
                             $this->lastDateHourly->format('%Y-%m-%d %H:%M:%S') . ".\n";
            $message = 'Maintenance statistics last updated hourly statistics to ' .
                       $this->lastDateHourly->format('%Y-%m-%d %H:%M:%S') . ".";
            MAX::debug($message, PEAR_LOG_DEBUG);
            $requiredDate = &new Date();
            $requiredDate->copy($this->lastDateHourly);
            $requiredDate->addSeconds(60 * 60);
            $this->report .= 'Current time must be after ' . $requiredDate->format('%Y-%m-%d %H:%M:%S') .
                             " for the next hourly update to happen.\n";
            $message = 'Current time must be after ' . $requiredDate->format('%Y-%m-%d %H:%M:%S') .
                       ' for the next hourly update to happen';
            MAX::debug($message, PEAR_LOG_DEBUG);
            if (Date::compare($this->now, $requiredDate) > 0) {
                $this->updateHourly = true;
                // Update hourly stats to the end of the previous (not current) hour
                $this->upToDateHourly = new Date($this->now->format('%Y-%m-%d %H:00:00'));
                $this->upToDateHourly->subtractSeconds(1);
            } else {
                // An hour hasn't passed, so don't update
                $this->report .= "At least 60 minutes have NOT passed since the last hourly update.\n";
                $message = 'At least 60 minutes have NOT passed since the last hourly update';
                MAX::debug($message, PEAR_LOG_DEBUG);
            }
        }
        if ($this->updateOI || $this->updateHourly) {
            $this->report .= "Maintenance statistics will be run.\n";
            $message = "Maintenance statistics will be run";
            MAX::debug($message, PEAR_LOG_INFO);
            if ($this->updateOI) {
                $this->report .= "The statistics will be updated on the basis of operation interval.\n";
                $message = 'The statistics will be updated on the basis of operation interval';
                MAX::debug($message, PEAR_LOG_INFO);
            }
            if ($this->updateHourly) {
                $this->report .= "The statistics will be updated on the basis of the hour.\n";
                $message = 'The statistics will be updated on the basis of the hour';
                MAX::debug($message, PEAR_LOG_INFO);
            }
            $this->report .= "\n";
        } else {
            $this->report .= "Maintenance statistics will NOT be run.\n\n";   
            $message = 'Maintenance statistics will NOT be run';
            MAX::debug($message, PEAR_LOG_INFO);
        }
    }
    
    /**
     * A method to summarise raw data into the intermediate statistics tables.
     *
     * @abstract
     */
    function summariseIntermediate() {}
    
    /**
     * A method to summarise intermediate statistics into summary (final) statistics
     * tables, where required.
     *
     * @abstract
     */
    function summariseFinal() {}

    /**
     * A method to delete raw data that is no-longer required, if the option to
     * do so is set in the configuration .ini file.
     */
    function deleteOldData()
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        if ($conf['maintenance']['compactStats']) {
            $time = time();            
            if (($this->updateHourly) && ($this->updateUsingOI)) {
                // Have updated the hourly stats, and the operation interval is less than or equal to an
                // hour, so it's safe to delete the statistics that have been summarised now
                $summarisedTo = &new Date();
                $summarisedTo->copy($this->upToDateHourly);
            } elseif (($this->updateOI) && (!$this->updateUsingOI)) {
                // Have updated the operation interval stats, and the operation interval is more than
                // an hour, so it's safe to delete the statistics that have been summarised now
                $summarisedTo = &new Date();
                $summarisedTo->copy($this->upToDateOI);
            }
            if (!is_null($summarisedTo)) {
                $rows = $this->dal->deleteOldData($summarisedTo);
                $time = time() - $time;
                if ($conf['table']['split']) {
                    $this->report .= "\nDropped $rows tables of old, summarised data in $time seconds.\n";
                    $message = "Dropped $rows tables of old, summarised data in $time seconds";
                } else {
                    $this->report .= "\nDeleted $rows rows of old, summarised data in $time seconds.\n";
                    $message = "Deleted $rows rows of old, summarised data in $time seconds";
                }
                MAX::debug($message, PEAR_LOG_DEBUG);
            }
        }
    }

    /**
     * A method to log the completion of summarising the raw data into statistics.
     *
     * @param Date $now Optional Date representing the current date/time. Normally
     *                  just used for testing the class constructor.
     */
    function logCompletion($now = null)
    {
        $end = &new Date();
        if (!is_null($now)) {
            $end->copy($now);
        }
        if (($this->updateHourly) && ($this->updateOI)) {
            $this->dal->logCompletion($this->now, $end, 2, $this->upToDateOI, $this->report);
        } elseif ($this->updateHourly) {
            $this->dal->logCompletion($this->now, $end, 1, $this->upToDateHourly, $this->report);
        } elseif ($this->updateOI) {
            $this->dal->logCompletion($this->now, $end, 0, $this->upToDateOI, $this->report);
        }
    }
}

?>
